//  IBM_PROLOG_BEGIN_TAG
//  This is an automatically generated prolog.
//
//  $Source: src/kernel/syscall.C $
//
//  IBM CONFIDENTIAL
//
//  COPYRIGHT International Business Machines Corp. 2010 - 2011
//
//  p1
//
//  Object Code Only (OCO) source materials
//  Licensed Internal Code Source Materials
//  IBM HostBoot Licensed Internal Code
//
//  The source code for this program is not published or other-
//  wise divested of its trade secrets, irrespective of what has
//  been deposited with the U.S. Copyright Office.
//
//  Origin: 30
//
//  IBM_PROLOG_END
#include <assert.h>
#include <errno.h>
#include <kernel/cpu.H>
#include <kernel/cpumgr.H>
#include <kernel/scheduler.H>
#include <kernel/taskmgr.H>
#include <kernel/task.H>
#include <kernel/syscalls.H>
#include <kernel/console.H>
#include <kernel/pagemgr.H>
#include <kernel/msg.H>
#include <kernel/timemgr.H>
#include <kernel/futexmgr.H>
#include <kernel/cpuid.H>
#include <kernel/misc.H>
#include <kernel/msghandler.H>
#include <kernel/vmmmgr.H>
#include <kernel/stacksegment.H>
#include <kernel/heapmgr.H>
#include <kernel/intmsghandler.H>
#include <sys/sync.h>

extern "C"
void kernel_execute_decrementer()
{
    cpu_t* c = CpuManager::getCurrentCPU();
    Scheduler* s = c->scheduler;
    TimeManager::checkReleaseTasks(s);
    s->returnRunnable();

    CpuManager::executePeriodics(c);//TODO is there still a potential deadlock?

    if (CpuManager::isShutdownRequested())
    {
        // The code below could cause a hang during shutdown
        // The stats can be retrieved from global variables as needed.
        // This can be uncommented for debug if desired
#ifdef __MEMSTATS__
        if(c->master)
            HeapManager::stats();
#endif
        KernelMisc::shutdown();
    }
    CpuManager::executePeriodics(c);
    s->setNextRunnable();
}

namespace Systemcalls
{
    typedef void(*syscall)(task_t*);
    void TaskYield(task_t*);
    void TaskStart(task_t*);
    void TaskEnd(task_t*);
    void TaskMigrateToMaster(task_t*);
    void TaskWait(task_t*);
    void MsgQCreate(task_t*);
    void MsgQDestroy(task_t*);
    void MsgQRegisterRoot(task_t*);
    void MsgQResolveRoot(task_t*);
    void MsgSend(task_t*);
    void MsgSendRecv(task_t*);
    void MsgRespond(task_t*);
    void MsgWait(task_t*);
    void MmioMap(task_t*);
    void MmioUnmap(task_t*);
    void DevMap(task_t*);
    void DevUnmap(task_t*);
    void TimeNanosleep(task_t*);
    void Futex(task_t *t);
    void Shutdown(task_t *t);
    void CpuCoreType(task_t *t);
    void CpuDDLevel(task_t *t);
    void MmAllocBlock(task_t *t);
    void MmRemovePages(task_t *t);
    void MmSetPermission(task_t *t);


    syscall syscalls[] =
    {
        &TaskYield,  // TASK_YIELD
        &TaskStart,  // TASK_START
        &TaskEnd,  // TASK_END
        &TaskMigrateToMaster, // TASK_MIGRATE_TO_MASTER
        &TaskWait, // TASK_WAIT

        &MsgQCreate,  // MSGQ_CREATE
        &MsgQDestroy,  // MSGQ_DESTROY
        &MsgQRegisterRoot,  // MSGQ_REGISTER_ROOT
        &MsgQResolveRoot,  // MSGQ_RESOLVE_ROOT

        &MsgSend,  // MSG_SEND
        &MsgSendRecv,  // MSG_SENDRECV
        &MsgRespond,  // MSG_RESPOND
        &MsgWait,  // MSG_WAIT
        &DevMap,  // DEV_MAP
        &DevUnmap,  // DEV_UNMAP

        &TimeNanosleep,  // TIME_NANOSLEEP

        &Futex,      // SYS_FUTEX operations

        &Shutdown,    // MISC_SHUTDOWN
        &CpuCoreType, // MISC_CPUCORETYPE
        &CpuDDLevel,  // MISC_CPUDDLEVEL

        &MmAllocBlock, // MM_ALLOC_BLOCK
        &MmRemovePages, // MM_REMOVE_PAGES
        &MmSetPermission, // MM_SET_PERMISSION
        };
};

extern "C"
void kernel_execute_system_call()
{
    using namespace Systemcalls;
    task_t* t = TaskManager::getCurrentTask();

    uint64_t syscall = t->context.gprs[3];
    if (syscall >= SYSCALL_MAX)
    {
        // TODO : kill task.
        printk("Invalid syscall : %ld\n", syscall);
        while(1);
    }
    else
    {
        syscalls[syscall](t);
    }
}

namespace Systemcalls
{
    void TaskYield(task_t* t)
    {
        Scheduler* s = t->cpu->scheduler;
        s->returnRunnable();
        s->setNextRunnable();
    }

    void TaskStart(task_t* t)
    {
        task_t* newTask =
            TaskManager::createTask((TaskManager::task_fn_t)TASK_GETARG0(t),
                                    (void*)TASK_GETARG1(t));
        newTask->cpu = t->cpu;
        t->cpu->scheduler->addTask(newTask);

        TASK_SETRTN(t, newTask->tid);
    }

    void TaskEnd(task_t* t)
    {
        TaskManager::endTask(t, (void*)TASK_GETARG0(t),
                             TASK_STATUS_EXITED_CLEAN);
    }

    void TaskMigrateToMaster(task_t* t)
    {
        // Move r6 to r3.
        //     This is needed so that this system call can be called from
        //     within a "fast" system call in start.S.  The fast system call
        //     will populate r6 with it's own syscall number.  When we return
        //     from this system call, on the master processor, we'll be back
        //     at the 'sc' instruction with r3 back to the fast syscall, and
        //     the fast syscall will be executed on the master processor.
        TASK_SETRTN(t, TASK_GETARG2(t));

        // Move task to master CPU and pick a new task.
        t->cpu->scheduler->addTaskMasterCPU(t);
        t->cpu->scheduler->setNextRunnable();
    }

    void TaskWait(task_t* t)
    {
        int64_t tid = static_cast<int64_t>(TASK_GETARG0(t));
        int* status = reinterpret_cast<int*>(TASK_GETARG1(t));
        void** retval = reinterpret_cast<void**>(TASK_GETARG2(t));

        // Validate status address and convert to kernel address.
        if (status != NULL)
        {
            uint64_t addr =
                VmmManager::findPhysicalAddress(
                    reinterpret_cast<uint64_t>(status));

            if (addr == (static_cast<uint64_t>(-EFAULT)))
            {
                TASK_SETRTN(t, -EFAULT);
                return;
            }
            status = reinterpret_cast<int*>(addr);
        }

        // Validate retval address and convert to kernel address.
        if (retval != NULL)
        {
            uint64_t addr =
                VmmManager::findPhysicalAddress(
                    reinterpret_cast<uint64_t>(retval));

            if (addr == (static_cast<uint64_t>(-EFAULT)))
            {
                TASK_SETRTN(t, -EFAULT);
                return;
            }
            retval = reinterpret_cast<void**>(addr);
        }

        // Perform wait.
        TaskManager::waitTask(t, tid, status, retval);
    }

    void MsgQCreate(task_t* t)
    {
        TASK_SETRTN(t, (uint64_t) new MessageQueue());
    }

    void MsgQDestroy(task_t* t)
    {
        MessageQueue* mq = (MessageQueue*) TASK_GETARG0(t);
        if (NULL != mq)
            delete mq;
        TASK_SETRTN(t, 0);
    }

    static MessageQueue* msgQRoot = NULL;
    static MessageQueue* msgQIntr = NULL;

    void MsgQRegisterRoot(task_t* t)
    {
        switch(TASK_GETARG0(t))
        {
            case MSGQ_ROOT_VFS:
                msgQRoot = (MessageQueue*) TASK_GETARG1(t);
                TASK_SETRTN(t,0);
                break;

            case MSGQ_ROOT_INTR:
                msgQIntr = (MessageQueue*) TASK_GETARG1(t);
                InterruptMsgHdlr::create(msgQIntr);
                TASK_SETRTN(t,0);
                break;

            default:
                printk("ERROR MsgRegisterRoot invalid type %ld\n",
                       TASK_GETARG0(t));
                TASK_SETRTN(t,-EINVAL);
        }
    }

    void MsgQResolveRoot(task_t* t)
    {
        switch(TASK_GETARG0(t))
        {
            case MSGQ_ROOT_VFS:
                TASK_SETRTN(t, (uint64_t) msgQRoot);
                break;

            case MSGQ_ROOT_INTR:
                TASK_SETRTN(t, (uint64_t) msgQIntr);
                break;

            default:
                printk("ERROR MsgQResolveRoot invalid type %ld\n",
                       TASK_GETARG0(t));
                TASK_SETRTN(t,0);
        }
    }

    void MsgSend(task_t* t)
    {
        MessageQueue* mq = (MessageQueue*) TASK_GETARG0(t);
        msg_t* m = (msg_t*) TASK_GETARG1(t);

        if ((NULL == mq) || (NULL == m))
        {
            printkd("NULL pointer for message queue (%p) or message (%p).\n",
                    mq, m);
            TASK_SETRTN(t, -EINVAL);
            return;
        }

        m->__reserved__async = 0; // set to async msg.

        if (m->type >= MSG_FIRST_SYS_TYPE)
        {
            printkd("Invalid type for msg_send, type=%d.\n", m->type);
            TASK_SETRTN(t, -EINVAL);
            return;
        }

        mq->lock.lock();

        // Get waiting (server) task.
        task_t* waiter = mq->waiting.remove();
        if (NULL == waiter) // None found, add to 'messages' queue.
        {
            MessagePending* mp = new MessagePending();
            mp->key = m;
            mp->task = t;
            mq->messages.insert(mp);
        }
        else // Add waiter back to its scheduler.
        {
            TASK_SETRTN(waiter, (uint64_t) m);
            waiter->cpu->scheduler->addTask(waiter);
        }

        mq->lock.unlock();
        TASK_SETRTN(t, 0);
    }

    void MsgSendRecv(task_t* t)
    {
        MessageQueue* mq = (MessageQueue*) TASK_GETARG0(t);
        msg_t* m = (msg_t*) TASK_GETARG1(t);
        MessageQueue* mq2 = (MessageQueue*) TASK_GETARG2(t);

        m->__reserved__async = 1; // set to sync msg.
        if (NULL != mq2) // set as pseudo-sync if secondary queue given.
        {
            m->__reserved__pseudosync = 1;
        }

        if (m->type >= MSG_FIRST_SYS_TYPE)
        {
            printkd("Invalid message type for msg_sendrecv, type=%d.\n",
                    m->type);
            TASK_SETRTN(t, -EINVAL);
            return;
        }

        // Create pending response object.
        MessagePending* mp = new MessagePending();
        mp->key = m;
        if (!m->__reserved__pseudosync) // Normal sync, add task to pending obj.
        {
            mp->task = t;
            t->state = TASK_STATE_BLOCK_MSG;
            t->state_info = mq;
        }
        else // Pseudo-sync, add the secondary queue instead.
        {
            mp->task = reinterpret_cast<task_t*>(mq2);
            TASK_SETRTN(t, 0);  // Need to give good RC for the caller, since
                                // we are returning immediately.
        }

        mq->lock.lock();

        // Get waiting (server) task.
        task_t* waiter = mq->waiting.remove();
        if (NULL == waiter) // None found, add to 'messages' queue.
        {
            mq->messages.insert(mp);
            if (!m->__reserved__pseudosync)
            {
                // Choose next thread to execute, this one is delayed.
                t->cpu->scheduler->setNextRunnable();
            } // For pseudo-sync, just keep running the current task.
        }
        else // Context switch to waiter.
        {
            TASK_SETRTN(waiter, (uint64_t) m);
            mq->responses.insert(mp);
            waiter->cpu = t->cpu;
            if (m->__reserved__pseudosync) // For pseudo-sync, add this task
                                           // back to scheduler.
            {
                t->cpu->scheduler->addTask(t);
            }
            TaskManager::setCurrentTask(waiter);
        }

        mq->lock.unlock();
    }

    void MsgRespond(task_t* t)
    {
        MessageQueue* mq = (MessageQueue*) TASK_GETARG0(t);
        msg_t* m = (msg_t*) TASK_GETARG1(t);

        mq->lock.lock();
        MessagePending* mp = mq->responses.find(m);
        if (NULL != mp)
        {
            task_t* waiter = mp->task;

            mq->responses.erase(mp);
            mq->lock.unlock();
            delete mp;

            // Kernel message types are handled by MessageHandler objects.
            if (m->type >= MSG_FIRST_SYS_TYPE)
            {
                TASK_SETRTN(t,
                            ((MessageHandler*)waiter)->recvMessage(m));

                if (TaskManager::getCurrentTask() != t)
                {
                    t->cpu->scheduler->addTask(t);
                }
            }
            // Pseudo-sync messages are handled by pushing the response onto
            // a message queue.
            else if (m->__reserved__pseudosync)
            {
                MessageQueue* mq2 = (MessageQueue*) waiter;
                mq2->lock.lock();

                // See if there is a waiting task (the original client).
                task_t* client = mq2->waiting.remove();
                if (NULL == client) // None found, add to queue.
                {
                    MessagePending* mp2 = new MessagePending();
                    mp2->key = m;
                    mp2->task = t;
                    mq2->messages.insert(mp2);
                }
                else // Add waiting task onto its scheduler.
                {
                    TASK_SETRTN(client, (uint64_t) m);
                    client->cpu->scheduler->addTask(client);
                }

                mq2->lock.unlock();
                TASK_SETRTN(t, 0);

            }
            // Normal-sync messages are handled by releasing the deferred task.
            else
            {
                waiter->cpu = t->cpu;
                TaskManager::setCurrentTask(waiter);
                TASK_SETRTN(waiter,0);

                TASK_SETRTN(t,0);
                t->cpu->scheduler->addTask(t);
            }
        }
        else
        {
            TASK_SETRTN(t, -EBADF);
            mq->lock.unlock();
        }
    }

    void MsgWait(task_t* t)
    {
        MessageQueue* mq = (MessageQueue*) TASK_GETARG0(t);

        mq->lock.lock();
        MessagePending* mp = mq->messages.remove();

        if (NULL == mp)
        {
            mq->waiting.insert(t);
            t->state = TASK_STATE_BLOCK_MSG;
            t->state_info = mq;
            t->cpu->scheduler->setNextRunnable();
        }
        else
        {
            msg_t* m = mp->key;
            if (m->__reserved__async)
                mq->responses.insert(mp);
            else
                delete mp;
            TASK_SETRTN(t, (uint64_t) m);
        }
        mq->lock.unlock();
    }

    /**
     * Map a device into virtual memory
     * @param[in] t:  The task used to map a device
     */
    void DevMap(task_t *t)
    {
        void *ra = (void*)TASK_GETARG0(t);
        uint64_t devDataSize = TASK_GETARG1(t);

        kassert(TASK_SETRTN(t, (uint64_t)VmmManager::devMap(ra,devDataSize)) !=
                NULL);
    }

    /**
     * Unmap a device from virtual memory
     * @param[in] t:  The task used to unmap a device
     */
    void DevUnmap(task_t *t)
    {
        void *ea = (void*)TASK_GETARG0(t);

        TASK_SETRTN(t, VmmManager::devUnmap(ea));
    }

    void TimeNanosleep(task_t* t)
    {
        TimeManager::delayTask(t, TASK_GETARG0(t), TASK_GETARG1(t));
        TASK_SETRTN(t, 0);

        t->cpu->scheduler->setNextRunnable();
    }


    void Futex(task_t * t)
    {
        uint64_t op = static_cast<uint64_t>(TASK_GETARG0(t));
        uint64_t futex = static_cast<uint64_t>(TASK_GETARG1(t));
        uint64_t val = static_cast<uint64_t>(TASK_GETARG2(t));
        uint64_t val2 = static_cast<uint64_t>(TASK_GETARG3(t));
        uint64_t futex2 = static_cast<uint64_t>(TASK_GETARG4(t));
        uint64_t rc = 0;

        // Set RC to success initially.
        TASK_SETRTN(t,0);

        futex = VmmManager::findPhysicalAddress(futex);
        if(futex == (static_cast<uint64_t>(-EFAULT)))
        {
            printk("Task %d terminated. No physical address found for address 0x%p",
                   t->tid,
                   reinterpret_cast<void *>(futex));

            TaskManager::endTask(t, NULL, TASK_STATUS_CRASHED);
            return;
        }

        uint64_t * futex_p = reinterpret_cast<uint64_t *>(futex);

        switch(op)
        {
            case FUTEX_WAIT: // Put task on wait queue based on futex

                rc = FutexManager::wait(t, futex_p, val);

                // Can only be set rc if control of the task is still had,
                // which is only, for certain, on error rc's
                if(rc != 0)
                {
                    TASK_SETRTN(t,rc);
                }
                break;

            case FUTEX_WAKE: // Wake task(s) on the futex wait queue

                rc = FutexManager::wake(futex_p, val);
                TASK_SETRTN(t,rc);
                break;

            case FUTEX_REQUEUE:
                // Wake (val) task(s) on futex && requeue remaining tasks on futex2

                futex2 = VmmManager::findPhysicalAddress(futex2);
                if(futex2 == (static_cast<uint64_t>(-EFAULT)))
                {
                    printk("Task %d terminated. No physical address found for address 0x%p",
                           t->tid,
                           reinterpret_cast<void *>(futex2));

                    TaskManager::endTask(t, NULL, TASK_STATUS_CRASHED);
                    return;
                }

                rc = FutexManager::wake(futex_p, val,
                                        reinterpret_cast<uint64_t *>(futex2),
                                        val2);
                break;

            default:
                printk("ERROR Futex invalid op %ld\n",op);
                TASK_SETRTN(t,static_cast<uint64_t>(-EINVAL));
        };
    }


    /**
     * Shutdown all CPUs
     * @param[in] t:  The current task
     */
    void Shutdown(task_t * t)
    {
        uint64_t status = static_cast<uint64_t>(TASK_GETARG0(t));
        CpuManager::requestShutdown(status);
        TASK_SETRTN(t, 0);
    }

    /** Read CPU Core type using CpuID interfaces. */
    void CpuCoreType(task_t *t)
    {
        TASK_SETRTN(t, CpuID::getCpuType());
    }

    /** Read CPU DD level using CpuID interfaces. */
    void CpuDDLevel(task_t *t)
    {
        TASK_SETRTN(t, CpuID::getCpuDD());
    }

    /**
     * Allocate a block of virtual memory within the base segment
     * @param[in] t: The task used to allocate a block in the base segment
     */
    void MmAllocBlock(task_t* t)
    {
        MessageQueue* mq = (MessageQueue*)TASK_GETARG0(t);
        void* va = (void*)TASK_GETARG1(t);
        uint64_t size = (uint64_t)TASK_GETARG2(t);

        TASK_SETRTN(t, VmmManager::mmAllocBlock(mq,va,size));
    }

    /**
     * Remove pages from virtual memory
     * @param[in] t: The task used to remove pages
     */
    void MmRemovePages(task_t* t)
    {
        VmmManager::PAGE_REMOVAL_OPS oper =
                (VmmManager::PAGE_REMOVAL_OPS)TASK_GETARG0(t);
        void* vaddr = (void*)TASK_GETARG1(t);
        uint64_t size = (uint64_t)TASK_GETARG2(t);

        TASK_SETRTN(t, VmmManager::mmRemovePages(oper,vaddr,size,t));
    }

     /**
     * Set the Permissions on a block containing the virtual address passed in.
     * @param[in] t: The task used to set Page Permissions for a given block
     */
    void MmSetPermission(task_t* t)
    {
        void* va = (void*)TASK_GETARG0(t);
        uint64_t size = (uint64_t)TASK_GETARG1(t);
        PAGE_PERMISSIONS access_type = (PAGE_PERMISSIONS)TASK_GETARG2(t);

        TASK_SETRTN(t, VmmManager::mmSetPermission(va,size, access_type));
    }

};

